@@ -7,6 +7,11 @@ from jinja2.ext import Extension
 
 from jac import compile
 
+try:
+    from collections import OrderedDict
+except ImportError:
+    from jac.compat.ordereddict import OrderedDict
+
 
 class CompilerExtension(Extension):
     tags = set(['compress'])
@@ -22,16 +27,20 @@ class CompilerExtension(Extension):
         return nodes.CallBlock(self.call_method('_compile', args), [], [], body).set_lineno(lineno)
 
     def _find_compilable_tags(self, soup):
-        for link in soup.find_all('link'):
-            yield link
-
-        for style in soup.find_all('style'):
-            yield style
-
-        for script in soup.find_all('script'):
-            yield script
+        tags = ['link', 'style', 'script']
+        for tag in soup.find_all(tags):
+            if tag.get('type') is None:
+                if tag.name == 'script':
+                    tag['type'] = 'text/javascript'
+                if tag.name == 'style':
+                    tag['type'] = 'text/css'
+            else:
+                tag['type'] == tag['type'].lower()
+            yield tag
 
     def _render_block(self, filename, type):
+        """Returns an html element pointing to filename as a string.
+        """
         filename = '%s/%s' % (self.environment.compressor_static_prefix, os.path.basename(filename))
 
         if type.lower() == 'css':
@@ -44,7 +53,6 @@ class CompilerExtension(Extension):
     def _find_file(self, path):
         if callable(self.environment.compressor_source_dirs):
             filename = self.environment.compressor_source_dirs(path)
-            print filename
             if os.path.exists(filename):
                 return filename
         else:
@@ -82,42 +90,73 @@ class CompilerExtension(Extension):
             return src
 
     def _compile(self, compression_type, caller):
-        compression_type = compression_type.lower()
         html = caller()
+
+        enabled = (not hasattr(self.environment, 'compressor_enabled') or
+                        self.environment.compressor_enabled is not False)
+        if not enabled:
+            return html
+
+        debug = (hasattr(self.environment, 'compressor_debug') and
+                      self.environment.compressor_debug is True)
+        compression_type = compression_type.lower()
         soup = BeautifulSoup(html)
         compilables = self._find_compilable_tags(soup)
-        text = ''
+        outdir = str(self.environment.compressor_output_dir)
+        static_prefix = str(self.environment.compressor_static_prefix)
+        assets = OrderedDict()
 
         html_hash = self._make_hash(html, self._find_compilable_tags(soup))
 
+        if not os.path.exists(str(self.environment.compressor_output_dir)):
+            os.makedirs(str(self.environment.compressor_output_dir))
+
         cached_file = os.path.join(str(self.environment.compressor_output_dir),
                                    '%s.%s') % (html_hash, compression_type)
 
         if os.path.exists(cached_file):
             return self._render_block(cached_file, compression_type)
 
+        count = 0
         for c in compilables:
             if c.get('type') is None:
                 raise RuntimeError('Tags to be compressed must have a compression_type.')
 
             src = c.get('src') or c.get('href')
             if src:
+                filename = os.path.basename(str(src)).split('.', 1)[0]
+                uri_cwd = os.path.join(static_prefix, os.path.dirname(str(src)))
                 src = open(self._find_file(str(src)), 'rb')
                 cwd = os.path.dirname(src.name)
             else:
+                uri_cwd = None
+                filename = 'inline{0}'.format(count)
                 src = c.string
                 cwd = None
 
-            if c.name == 'link' and c.get('rel', [''])[0].lower() != 'stylesheet':
-                text += self._get_contents(src)
-
-            if c['type'].lower() in ('text/css', 'text/javascript'):
-                text += self._get_contents(src)
+            needs_compile = c['type'] != 'text/javascript'
+            if not debug or needs_compile:
+                text = compile(self._get_contents(src), c['type'], cwd=cwd,
+                               uri_cwd=uri_cwd, debug=debug)
             else:
-                text += compile(self._get_contents(src), c['type'], cwd=cwd)
+                text = self._get_contents(src)
 
-        with open(cached_file, 'w') as f:
-            f.write(text)
+            if not debug:
+                outfile = cached_file
+            else:
+                outfile = os.path.join(outdir, '%s-%s.%s') % (html_hash,
+                          filename, compression_type)
 
-        return self._render_block(cached_file, compression_type)
+            if assets.get(outfile) is None:
+                assets[outfile] = ''
+            assets[outfile] += text
 
+            count += 1
+
+        blocks = ''
+        for outfile, asset in assets.items():
+            with open(outfile, 'w') as f:
+                f.write(asset)
+            blocks += self._render_block(outfile, compression_type)
+
+        return blocks
